import java.awt.Container;
import java.awt.Graphics;
import java.awt.Color;
/**
  *
  * Beschreibung
  *
  * @version 1.0 vom 10.04.2011
  * @author Daniel Garmann
  */

public class HeckmeckSpiel {

  // Anfang Attribute
  private int spieleranzahl;
  private int amZug = 0;
  private Karte[] kartenspiel = new Karte[16];
  private Stack[] spielerstapel;
  
  private Wuerfel[] wuerfel = new Wuerfel[8];
  

  // Ende Attribute

  // Anfang Methoden
  public HeckmeckSpiel(int spieleranzahl) {
    this.spieleranzahl = spieleranzahl;
    spielerstapel = new Stack[spieleranzahl];
    for (int i = 0; i < spieleranzahl; i++) {
      spielerstapel[i] = new Stack();
    }
//    for (int i = 0; i < 2; i++) {
    for (int i = 0; i < 16; i++) {
      kartenspiel[i] = new Karte(i+21, i / 4 + 1);
    }
    for(int i = 0; i < 8; i++) {
      wuerfel[i] = new Wuerfel();
    }
  }
  
  public void wuerfeln() {
    for (int i = 0; i < 8; i++) {
      wuerfel[i].wuerfeln();
    }
  }
  
  public boolean rausnehmen(int zahlenwert) {
    for (int i = 0; i < 8; i++) {
      if (wuerfel[i].getWert() == zahlenwert && wuerfel[i].getFest() == true) { // Zahlenwert wurde schon rausgenommen
        return false;
      }
    }
    boolean vorhanden = false;
    for (int i = 0; i < 8; i++) {
      if (wuerfel[i].getWert() == zahlenwert) {
        wuerfel[i].feststellen();
        vorhanden = true;
      }
    }
    return vorhanden;
  }
  
  public int augensumme() {
    int summe = 0;
    for (int i = 0; i < 8; i++) {
      if (wuerfel[i].getFest() == true) {
        summe = summe + wuerfel[i].getWert();
        if (wuerfel[i].getWert() == 6) {
          summe = summe - 1;
        }
      }
    }
    return summe;
  }

  public boolean wurmvorhanden() {
    boolean wurmvorhanden = false;
    for (int i = 0; i < 8; i++) {
      if (wuerfel[i].getFest() == true && wuerfel[i].getWert() == 6) {
        wurmvorhanden = true;
      }
    }
    return wurmvorhanden;
  }
  
  public void spielerwechsel() {
    amZug++;
    if (amZug == spieleranzahl) {
      amZug = 0;
    }
    for (int i = 0; i < 8; i++) {
      wuerfel[i].loslassen();
    }
  }
  
  public int getAmZug() {
    return amZug;
  }
  
  public int spieleranzahl() {
    return spieleranzahl;
  }

  public boolean kartenehmen(int kartenwert) {
    if (augensumme() < kartenwert || wurmvorhanden() == false) { // augensumme reicht nicht fr karte
      return false;
    }

    if (kartenspiel[kartenwert-21] != null) { // karte liegt noch offen aus
      spielerstapel[amZug].push(kartenspiel[kartenwert-21]);
      kartenspiel[kartenwert-21] = null;
      return true;
    }
    // karte liegt evtl. offen auf den Spielerstapeln
    for (int i = 0; i < spieleranzahl; i++) {
      if (!spielerstapel[i].isEmpty() && i != amZug && ((Karte)spielerstapel[i].top()).getKartenwert() == kartenwert && kartenwert == augensumme()) {
        spielerstapel[amZug].push(spielerstapel[i].top());
        spielerstapel[i].pop();
        return true;
      }
    }
    // karte lag nicht offen auf spielerstapel
    return false;
  }
  
  public void karteverlieren() {
    if (!spielerstapel[amZug].isEmpty()) {
      Karte k = (Karte)spielerstapel[amZug].top();
      spielerstapel[amZug].pop();
      kartenspiel[k.getKartenwert()-21] = k;
    }
    for (int i = 15; i >= 0; i--) {
      if (kartenspiel[i] != null) {
        kartenspiel[i] = null;
        return;
      }
    }
  }
  
  public int spielende() {
    for (int i = 0; i < 16; i++) {
      if (kartenspiel[i] != null) return -1;
    }
    return gewinner();
  }
  
  public int gewinner() {
    int result = -1;
    int maxsumme = -1;
    for (int i = 0; i < spieleranzahl; i++) {
      int summe = 0;
      Stack hilf = new Stack();
      while (!spielerstapel[i].isEmpty()) {
        Karte k = (Karte)spielerstapel[i].top();
        summe += k.getWurmwert();
        hilf.push(k);
        spielerstapel[i].pop();
      }
      while (!hilf.isEmpty()) {
        spielerstapel[i].push(hilf.top());
        hilf.pop();
      }
      if (summe > maxsumme) {
        maxsumme = summe;
        result = i;
      }
    }
    return result;
  }
  
  public String toString(){
    // Spieler am Zug + Trennsymbol
    String s = "" + amZug + ";" + spieleranzahl + "-";

    // offen liegende Karten
    for (int i = 0; i < 16; i++){
      if (i > 0) s += ";";
      if (kartenspiel[i] == null)
        s += "0";
      else
        s += kartenspiel[i].getKartenwert();
    }
    
    s += "-"; // Trennsymbol
    
    // Kartenstapel der Mitspieler
    for (int i = 0; i < spieleranzahl; i++) {
      if (i > 0) s += ";";
      if (spielerstapel[i].isEmpty())
        s +="0";
      else
        s += ((Karte)spielerstapel[i].top()).getKartenwert();
    }
    
    s += "-"; // Trennsymbol
    
    // Wrfel
    for (int i = 0; i < 8; i++) {
      if (i > 0) s += ";";
      s += wuerfel[i].getSymbol() + "" + wuerfel[i].getFest();
    }

    s += "-"; // Trennsymbol
    // Augensumme
    s += augensumme();
    
    return s;
  }

/*
  public void anzeigen(Container c) {
    Graphics g = c.getGraphics();
    g.clearRect(0,0,c.getWidth(),c.getHeight());
    // Karten zeigen
    for (int i = 0; i < 16; i++) {
      if (kartenspiel[i] != null) {
        g.drawRect(c.getWidth()/16 * i,20,c.getWidth()/17,40);
        g.drawString(""+kartenspiel[i].getKartenwert(),c.getWidth()/16*i+10,30);
        g.drawString(""+kartenspiel[i].getWurmwert(),c.getWidth()/16*i+10,50);
      } else {
        g.fillRect(c.getWidth()/16 * i,20,c.getWidth()/17,40);
      }
    }

    // Spielerstapel zeigen
    for (int i = 0; i < spieleranzahl; i++) {
      if (i == amZug) {
        g.setColor(Color.GREEN);
      } else {
        g.setColor(Color.BLACK);
      }
      if (spielerstapel[i].isEmpty() == false) {
        g.drawRect(c.getWidth()/spieleranzahl * i,80,c.getWidth()/17,40);
        g.drawString(""+((Karte)spielerstapel[i].top()).getKartenwert(),c.getWidth()/spieleranzahl*i+10,100);
        g.drawString(""+((Karte)spielerstapel[i].top()).getWurmwert(),c.getWidth()/spieleranzahl*i+10,120);
      } else {
        g.fillRect(c.getWidth()/spieleranzahl * i,80,c.getWidth()/17,40);
      }
    }
    
    // Wuerfel zeigen
    for (int i = 0; i < 8; i++) {
      if (wuerfel[i].getFest() == true) {
        g.setColor(Color.RED);
      } else {
        g.setColor(Color.BLACK);
      }
      g.drawRect(c.getWidth()/8 * i, 150, c.getWidth()/9, c.getWidth()/9);
      if (wuerfel[i].getWert() == 6) {
        g.drawString("W",c.getWidth()/8*i+20,180);
      } else {
        g.drawString(""+wuerfel[i].getWert(),c.getWidth()/8*i+20,180);
      }
    }
    g.drawString("Augensumme: "+augensumme(),10,170 + c.getWidth()/9);
  }
*/

  // Ende Methoden
}
