import javax.swing.*;
import java.util.ArrayList;
import java.awt.*;
import java.awt.event.*;
/**
  *
  * Beschreibung
  *
  * @version 1.0 vom 16.05.2011
  * @author Daniel Garmann
  */

public class SpielServer extends Server {

  // Anfang Attribute
  private TeilnehmerListe spieler = new TeilnehmerListe();
  private HeckmeckSpiel spiel;
  private boolean spielabbruchAbfragen = false;
  // Ende Attribute

  public SpielServer(int port) {
    super(port);
  }

  // Anfang Methoden




  public void processNewConnection(String pClientIP, int pClientPort) {
    send(pClientIP, pClientPort, "NAME");
  }


  public void processClosedConnection(String pClientIP, int pClientPort) {
    spieler.entfernen(pClientIP, pClientPort);
  }

  public void processMessage(String pClientIP, int pClientPort, String pMessage) {
    System.out.println("Server empfngt: "+pMessage);
    if (pMessage.startsWith("NAME")) {
      if (spieler.hinzufuegen(pClientIP, pClientPort, pMessage.substring(5))) {
        send(pClientIP, pClientPort, "OK01 Erfolgreich angemeldet!");
        sendToAll("SEND "+pMessage.substring(5)+" ist angemeldet!");
      } else {
        send(pClientIP, pClientPort, "ER01 Name vorhanden");
      }
    } else if (pMessage.startsWith("SEND")) {
      String nick = spieler.gibName(pClientIP, pClientPort);
      if (nick != null) {
        this.sendToAll("SEND "+nick+": "+pMessage.substring(5));
      }
    } else if (pMessage.startsWith("WHIS")) {
      String nick = spieler.gibName(pClientIP, pClientPort);
      pMessage = pMessage.substring(5)+" ";
      String[] msg = pMessage.split(":");
      String tonick = msg[0];
      int port = spieler.gibPort(tonick);
      String ip = spieler.gibIP(tonick);
      if (port != -1) {
        this.send(ip, port, "WHIS "+nick+":"+msg[1]);
      }
      this.send(pClientIP, pClientPort, "SEND Du hast zu '"+tonick+"' geflstert: "+pMessage.substring(5+tonick.length()));
    } else if (pMessage.startsWith("NICK")) {
      this.send(pClientIP,pClientPort,"NICK "+spieler.toString());
    } else if (pMessage.startsWith("DATA")) {
      if (spiel != null) {
        this.send(pClientIP,pClientPort,"DATA "+spiel.toString());
      }
    } else if (pMessage.startsWith("JOIN")) {
      String nick = spieler.gibName(pClientIP, pClientPort);
      if (spiel != null) {
        int nr = spieler.spielbeitreten(pClientIP, pClientPort);
        if (nr > spiel.spieleranzahl()) {
          spieler.spielverlassen(pClientIP, pClientPort);
          this.send(pClientIP, pClientPort, "ER02 Spiel schon gestartet");
        } else {
          send(pClientIP, pClientPort, "OK02");
          sendToAll("SEND "+nick+" ist dem Spiel beigetreten");
        }
      } else {
        spieler.spielbeitreten(pClientIP, pClientPort);
        send(pClientIP, pClientPort, "OK02");
        sendToAll("SEND "+nick+" ist dem Spiel beigetreten");
      }
    } else if (pMessage.startsWith("PLAY")) {
      if (spiel == null) {
        spiel = new HeckmeckSpiel(spieler.spieleranzahl());
        send(pClientIP, pClientPort, "OK03");
        sendToAll("DATA "+spiel.toString());
      } else {
        send(pClientIP, pClientPort, "ER03 Spielstart nicht mglich");
      }
    } else if (pMessage.startsWith("TOSS")) {
      Teilnehmer t = spieler.gibTeilnehmer(pClientIP, pClientPort);
      if (spiel != null && t != null && t.getSpielernr() == spiel.getAmZug() && spielabbruchAbfragen == false) {
        spiel.wuerfeln();
        send(pClientIP, pClientPort, "OK04");
        sendToAll("DATA "+spiel.toString());
      } else {
        this.send(pClientIP, pClientPort, "ER04 Wurf nicht mglich");
      }
    } else if (pMessage.startsWith("SAVE")) {
      Teilnehmer t = spieler.gibTeilnehmer(pClientIP, pClientPort);
      if (spiel != null && t != null && t.getSpielernr() == spiel.getAmZug() && spielabbruchAbfragen == false) {
        int wert = Integer.parseInt(pMessage.substring(5));
        if (wert == -1) {
          spiel.karteverlieren();
          if (spiel.spielende()>=0) {
            sendToAll("DATA "+spiel.toString());
            sendToAll("ENDE "+spieler.gibTeilnehmer(spiel.gewinner()).getNick());
            spiel = null;
          } else {
            spiel.spielerwechsel();
            send(pClientIP, pClientPort, "OK05");
            sendToAll("DATA "+spiel.toString());
          }
        } else if (spiel.rausnehmen(wert) == true) {
          send(pClientIP, pClientPort, "OK05");
          sendToAll("DATA "+spiel.toString());
        } else {
          this.send(pClientIP, pClientPort, "ER05 Auswahl nicht mglich");
        }
      } else {
        this.send(pClientIP, pClientPort, "ER05 Auswahl nicht mglich");
      }
    } else if (pMessage.startsWith("TAKE")) {
      Teilnehmer t = spieler.gibTeilnehmer(pClientIP, pClientPort);
      if (spiel != null && t != null && t.getSpielernr() == spiel.getAmZug() && spielabbruchAbfragen == false) {
        int wert = Integer.parseInt(pMessage.substring(5));
        if (spiel.kartenehmen(wert) ) {
          if (spiel.spielende()>=0) {
            sendToAll("DATA "+spiel.toString());
            sendToAll("ENDE "+spieler.gibTeilnehmer(spiel.gewinner()).getNick());
            spiel = null;
          } else {
            spiel.spielerwechsel();
            send(pClientIP, pClientPort, "OK06");
            sendToAll("DATA "+spiel.toString());
          }
        } else {
          this.send(pClientIP, pClientPort, "ER06 Auswahl nicht mglich");
        }
      } else {
        this.send(pClientIP, pClientPort, "ER06 Auswahl nicht mglich");
      }
    } else if (pMessage.startsWith("LIST")) {
      this.send(pClientIP,pClientPort,"LIST "+spieler.spielerliste());
    } else if (pMessage.startsWith("RSET")) {
      Teilnehmer t = spieler.gibTeilnehmer(pClientIP, pClientPort);
      if (t != null) {
        this.sendToAll("RSET "+t.getNick());
//        t.setReset(true);
        spielabbruchAbfragen = true;
      }
    } else if (pMessage.startsWith("YRST")) {
      Teilnehmer t = spieler.gibTeilnehmer(pClientIP, pClientPort);
      if (t != null && t.getSpielernr() >= 0) {
        t.setReset(true);
        if (spieler.allReset()) {
          spieler.setAllReset(false);
          this.sendToAll("ENDE "+spieler.gibTeilnehmer(spiel.gewinner()).getNick()+": Spielabbruch");
          spiel = null;
          spielabbruchAbfragen = false;
        }
        send(pClientIP, pClientPort, "OK07");
      }
    } else if (pMessage.startsWith("NRST")) {
      Teilnehmer t = spieler.gibTeilnehmer(pClientIP, pClientPort);
      if (t != null && t.getSpielernr()>= 0) {
        spieler.setAllReset(false);
        this.sendToAll("DATA "+spiel.toString());
        send(pClientIP, pClientPort, "OK07");
        spielabbruchAbfragen = false;
      }
    }
  }

  // Ende Methoden
}
